@@ -10,10 +10,12 @@ import yaml
 
 
 class MeasurementTree:
-    def __init__(self, filepath, index=True, override: bool = False):
+    def __init__(self, filepath, index=True, override: bool = False, chunk=None, keep_file_open=False):
         """
         :param filepath: r-string that points to h5 file
         :param index: optional: tuple that describes group number and group internal number
+        :param chunk: None will skip chunking before saving, True will set chunks to 'auto', dict will forward dict to xarray.Dataset.chunk call
+        :param keep_file: If False (default), the file will be closed after conversion.
         """
         self.filepath = filepath
         self.path = pl.Path(filepath).absolute()
@@ -23,6 +25,7 @@ class MeasurementTree:
         self.tree = [[]]
         self.indent_max = 0
         self.new_tree = []
+        self.chunk = chunk
         self.target = None
         self.shape = None
         self.data = None
@@ -57,10 +60,19 @@ class MeasurementTree:
             if index is False:
                 return
             self.save_netcdf()
+        if not keep_file_open:
+            self.f.close()
 
     def list_hdf5(self):
         print(self.f.visit(print))
 
+    @staticmethod
+    def save_file_from_string(obj, path):
+        try:
+            obj.to_netcdf(path)
+        except PermissionError:
+            obj.to_netcdf(str(path).encode('UTF-8'))
+
     def save_netcdf(self):
         self.array: xr.DataArray
         if self.index is not True:
@@ -70,10 +82,15 @@ class MeasurementTree:
         else:
             self.save_path = self.path.with_suffix('.nc').absolute()
         if self.dataset is not None:
-            self.dataset.to_netcdf(self.save_path)
+            if self.chunk is not None:
+                if isinstance(self.dataset, dict):
+                    self.dataset = self.dataset.chunk(**self.chunk)
+                elif self.chunk is True:
+                    self.dataset = self.dataset.chunk(chunks='auto')
+            self.save_file_from_string(self.dataset, self.save_path)
             print('Saved dataset as {}'.format(self.save_path))
         elif self.array is not None:
-            self.array.to_netcdf(self.save_path)
+            self.save_file_from_string(self.dataset, self.save_path)
             print('Saved as {}'.format(self.save_path))
 
 
@@ -208,7 +225,7 @@ class MeasurementTree:
                     except KeyError:
                         pass
                 try:
-                    if k['function'] == 'indicator':
+                    if k['function'] in ['indicator', 'internal - numeric input']:
                         possible_indicators.append((group, row))
                 except KeyError:
                     pass
@@ -297,7 +314,7 @@ class MeasurementTree:
                 u['control name'] = key_control_name
                 control_keys.append(key_control_name)
                 u['units'] = unit
-
+            print(u)
         # print([u['units'] for u in self.definition.values() if 'units' in u])
 
         all_indicators = []
@@ -307,7 +324,6 @@ class MeasurementTree:
             # self.index = x
             self.target: Group = self.new_tree[group]
 
-
             try:
                 self.data = self.target.get_data(row)
             except KeyError:
@@ -319,7 +335,13 @@ class MeasurementTree:
 
             global_row = self.target[row][0]
 
-            self.indicator_name = self.definition[global_row]['control name']
+            try:
+                self.indicator_name = self.definition[global_row]['control name']
+            except KeyError:
+                if self.definition[global_row]['function'] == 'internal - numeric input':
+                    self.indicator_name = self.definition[global_row]['name']
+                else:
+                    raise KeyError(f'Error while trying to fetch the name for an numeric input in {global_row}')
             print()
             print("Building xarray object for:")
             print(f"{self.indicator_name}, {global_row}")
@@ -364,13 +386,17 @@ class MeasurementTree:
                         print(f'Scalar control {control_name} without data. Generating coords.')
                         if isinstance(self.definition[parent_row]['start'], list):
                             part = []
+                            roi = []
+                            # TODO: Find a way to save this information for later use. Save list of steps in array.
                             for sta, sto, ste, eq in zip(self.definition[parent_row]['start'],
                                                          self.definition[parent_row]['stop'],
                                                          self.definition[parent_row]['steps'],
                                                          self.definition[parent_row]['equation']):
                                 part.append(np.linspace(sta, sto, int(ste)))
+                                roi.append(int(ste))
                                 row_data = np.concatenate(part)
                         else:
+                            roi = [int(self.definition[parent_row]['steps'])]
                             row_data = np.linspace(self.definition[parent_row]['start'],
                                                    self.definition[parent_row]['stop'],
                                                    int(self.definition[parent_row]['steps']))
@@ -417,6 +443,7 @@ class MeasurementTree:
             try:
                 self.data = np.reshape(self.data, self.shape)
             except ValueError:
+                # TODO: Find a way to drop empty slices
                 print('Measurement not finished?')
                 print(f'Desired shape: {self.shape}, data shape: {self.data.shape}, size: {self.data.size}')
                 flattened_length = np.prod(shape)
@@ -426,9 +453,12 @@ class MeasurementTree:
                 print(f'Number of core measurements: {flattened_length_data} of {flattened_length}')
                 flattened_shape = (flattened_length,)+data_shape
                 print(f'flattened shape: {flattened_shape}')
+                # initialize flattened with nan
                 flattened_new = np.empty(flattened_shape)
                 flattened_new[...] = np.nan
+                # write existing override nans with existing data
                 flattened_new[0:flattened_length_data, ...] = self.data
+                # bring data to correct shape
                 self.data = np.reshape(flattened_new, self.shape)
 
 
@@ -673,7 +703,8 @@ def consolidate_dims(array, name_includes, compare_to: str or None = None, new_d
     array = array.rename({temp_name: new_dim})
     return array
 
-
-
+# Add consolidate dims as methods to xarray.DataArray and xarray.Dataset
+xr.DataArray.consolidate_dims = consolidate_dims
+xr.Dataset.consolidate_dims = consolidate_dims
 
 
